pro shapelets_recomp, decomp, recomp, $
                      NRAN=nran,      $
                      TOP=top,        $
                      BOTTOM=bottom,  $
                      NOOVER=noover,  $
                      SKY=sky

;$Id: shapelets_recomp.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_RECOMP
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Compute the recomposed image corresponding to a set of shapelet
;      coefficients calculated using shapelets_decomp.pro. The input decomp
;      structure also contains a few mea-parameters, including the shapelet
;      scale size beta, and whether or not the basis functions should be
;      integrated within pixels, or merely evaluated at the centre of each
;      pixel.
;
; INPUTS:
;      decomp - structure produced by shapelets_decomp.pro
;
; OPTIONAL INPUTS:
;      nran   - Integer array [n_low,n_high]. Ignore coefficients with n 
;               outsdide this range.
;      top    - Integer. Use only this number of (Cartesian) shapelet 
;               coefficients during recomposition, selected as the ones with 
;               the largest absolute values.
;      bottom - Integer. Use only this number of (Cartesian) shapelet 
;               coefficients during recomposition, selected as the ones with 
;               the smallest absolute values.
;
; KEYWORD PARAMETERS:
;      /NOOVER- If set, do not oversample the basis functions.
;      /SKY   - If set, include fit to sky background in reconstructed image.
;
; OUTPUTS:
;      recomp - Reconstructed (2D floating point) image array.
;
; TO DO:
;      Could simplify sky replacement etc. by using shapelets_make_ls_matrix.pro
;
; MODIFICATION HISTORY:
;      Apr 04 - Header tidied up by RM.
;      Nov 01 - Modified by R. Massey to allow numerical integration
;               of basis functions
;      Feb 01 - Modified by AR to allow oversampling of the basis functions
;      Jul 99 - Written by A. Refregier
;-

; declarations
if not keyword_set(nran) then nran = [0,decomp.n_max]
shapelets_make_nvec,decomp.n_max,n1,n2,n_coeffs

; find top coefficients, if requested
if keyword_set(top) then begin
  sorted=reverse(sort(abs(decomp.coeffs)))
  coeffs_top=abs(decomp.coeffs(sorted(top-1)))
endif else begin
  coeffs_top=0.
endelse
if keyword_set(bottom) then begin
  sorted=sort(abs(decomp.coeffs))
  coeffsbottom=abs(decomp.coeffs(sorted(bottom-1)))
endif else begin
  coeffs_bottom=9.e99
endelse

; oversample if necessary
if keyword_set(noover) then ov=1 else ov=decomp.over
ovf=float(ov)

; create x arrays
shapelets_make_xarr, [decomp.n_pixels(0)*ov, decomp.n_pixels(1)*ov], x1, x2, x0=decomp.x*ovf

; construct the basis functions
Basis=shapelets_phi([decomp.n_max,0], x1/(decomp.beta*ovf), x2/(decomp.beta*ovf), $
      integrate=decomp.integrate, /array) / (decomp.beta*ovf)

; reconstruct the image by summing the (weighted) basis functions
recomp = fltarr(decomp.n_pixels*ov)
for i=0,n_coeffs-1 do begin
  n=n1[i]+n2[i]
  if n ge nran[0] and n le nran[1] and abs(decomp.coeffs[i]) ge coeffs_top  $
      and abs(decomp.coeffs[i]) le coeffs_bottom then $
      recomp = recomp + decomp.coeffs[i]*Basis[*,*,i]
endfor

; resample --- er, does this work?
if ov ne 1 then recomp=rebin(recomp,decomp.n_pixels[0],decomp.n_pixels[1])

; add sky background
if keyword_set(sky) then begin                          ; 
  recomp=recomp+decomp.skyfit[0]                        ; Add sky background
  if n_elements(decomp.skyfit) gt 1 then begin          ; 
    recomp=recomp+decomp.skyfit[1]*x1/(decomp.beta*ovf) ; Add sky gradient
    recomp=recomp+decomp.skyfit[2]*x2/(decomp.beta*ovf) ;
  endif                                                 ; 
endif  
  
end










