;$Id: shapelets_convolution_matrix.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ***********************************************************************
; ***********************************************************************

function shapelets_b3_tensor,alpha,beta,gamma,nmax=nmax

; Mar 02 - nmax keyword added by R. Massey but should be backwardly compatible
; Feb 01 - Written by A. Refregier
;
; PURPOSE: compute the 3-product tensor B^(3)_lmn(alpha,beta,gamma).
; This matrix is involved in convolutions, noise estimates and deprojections.
; INPUT: lmn=[l,m,n]: oscillator index
;        [alpha,beta,gamma]: scale lengths
;        nmax: size of array. Unfortunately has to be square at the moment but
;              would be quicker to let the size be different in n,m,l directions
; OUTPUT: shapelets_make_b3_tensor: B^(3)_lmn(alpha,beta,gamma)

; declarations
;l=lmn(0) & m=lmn(1) & n=lmn(2)
nu=(alpha^(-2)+beta^(-2)+gamma^(-2))^(-.5)
a=sqrt(2.)*nu/alpha
b=sqrt(2.)*nu/beta
c=sqrt(2.)*nu/gamma
if not keyword_set(nmax) then nmax=11

; construct B^(3) tensor
bt=fltarr(nmax+1,nmax+1,nmax+1)
p=fltarr(nmax+1,nmax+1,nmax+1) ;normalisation factors

for i=0,nmax do begin
 for j=0,nmax do begin
  for k=0,nmax do begin
   p(i,j,k)=nu*(2.^(i+j+k-1)*3.1415263^.5*factorial(i)*factorial(j)*factorial(k)*alpha*beta*gamma)^(-.5)
  endfor
 endfor
endfor


bt(0,0,0)=1.

for i=0,nmax do begin
 for l=0,i do begin
  for m=0,i do begin
   for n=0,i do begin
   
    if (l ge m) and (l ge n) then begin
	c1=0
	c2=0
	c3=0
	if ((l-1.) ge 0.) and ((l-2.) ge 0.) then c1=2.*(l-1.)*(a^2-1.)*bt(l-2,m,n)
	if ((l-1.) ge 0.) and ((m-1.) ge 0.) then c2=2.*m*a*b*bt(l-1,m-1,n)
	if ((l-1.) ge 0.) and ((n-1.) ge 0.) then c3=2.*n*a*c*bt(l-1,m,n-1)
    endif
    
    if (m ge l) and (m ge n) then begin
	c1=0
	c2=0
	c3=0
	if ((m-1.) ge 0.) and ((m-2.) ge 0.) then c1=2.*(m-1.)*(b^2-1.)*bt(l,m-2,n)
	if ((m-1.) ge 0.) and ((l-1.) ge 0.) then c2=2.*l*a*b*bt(l-1,m-1,n)
	if ((m-1.) ge 0.) and ((n-1.) ge 0.) then c3=2.*n*b*c*bt(l,m-1,n-1)
    endif
    
    if (n ge l) and (n ge m) then begin
	c1=0
	c2=0
	c3=0
	if ((n-1.) ge 0.) and ((n-2.) ge 0.) then c1=2.*(n-1.)*(c^2-1.)*bt(l,m,n-2)
	if ((n-1.) ge 0.) and ((l-1.) ge 0.) then c2=2.*l*a*c*bt(l-1,m,n-1)
	if ((n-1.) ge 0.) and ((m-1.) ge 0.) then c3=2.*m*b*c*bt(l,m-1,n-1)
    endif
    
    bt(l,m,n)=(c1+c2+c3)
    
    if (l eq 0) and (m eq 0) and (n eq 0) then bt(l,m,n)=1.
   
   endfor
  endfor
 endfor
endfor

for l=0,nmax do begin
 for m=0,nmax do begin
  for n=0,nmax do begin
   bt(l,m,n)=bt(l,m,n)*p(l,m,n)
  endfor
 endfor
endfor

return,bt

end






; ***********************************************************************
; ***********************************************************************

function shapelets_convolution_matrix,psf,alpha,n_max_a,gamma,n_max_g

;+
; Mar 02 - Written by R. Massey
;
; PURPOSE: Create an array of which unconvolved basis functions (with scale
;          alpha) combine to create a basis function convolved with a (decomp
;          structure) PSF (and scale gamma).
;
;          CBF_m(x;a) = Sigma(n=0->inf) { P_nm * BF_n(x;g) }
;       where    P_nm = Sigma(l=0->n_max_b) { C_nml(a,b,g) * g_l(b) }
;
; INPUTS:    psf: Cartesian decomp structure representing the local PSF
;        alpha: Scale size of unsmeared object
;        n_max_a: Original BFs go up to this n_max
;        n_max_g: Convolved BFs will be made up of unconvoled BFs up to this
;                 large number (ideally infinite)
;          gamma: Scale size of convolved basis functions
; OUTPUTS:  P_nm: "PSF matrix" of PaperII eqn 12
;                 Each column represents which BFs(g) a BF(a) transforms into
;                 under convolution with a PSF(b)
;-

; Original, unsmeared image f(x;a) or basis functions properties.
;alpha is read in from from command line (this is the beta we typically want to find)
;n_max_a is read in from command line (this is the n_max we typically want, or set)
shapelets_make_nvec, n_max_a, n1_a, n2_a, nn_a

; PSF g(x;b) properties. Counting variable is l.
beta=psf.beta
n_max_b=psf.n_max
shapelets_make_nvec, n_max_b, n1_b, n2_b, nn_b
g=psf.coeffs

; Convolved, observed image h(x;g) properties. Counting variable is n.
if n_params() eq 4 then begin $
  gamma=sqrt(alpha^2+beta^2) ; "Natural choice" in paperII
; gamma=alpha                ; Alternative choice, gives a symmetric P_nm
endif
;n_max_g (an intermediate step only) could in principle be very high, since no noise
shapelets_make_nvec, n_max_g, n1_g, n2_g, nn_g


;-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+


; Construct 1-dimensional convolution matrix (PaperII eqn 7)
bthree=shapelets_b3_tensor(1./gamma,1./alpha,1./beta,nmax=max([n_max_a,n_max_b,n_max_g])+1)
c=fltarr(n_max_g+1,n_max_a+1,n_max_b+1)
; surely c should be complex! Not sure why this works, but it does reproduce...
for n=0,n_max_g do begin ; c(i,j,k)=(2.*!dpi)^.5*(-1.)^i*complex(0,1)^(i+j+k)*bt(i,j,k)
  for m=0,n_max_a do begin ; exactly - well done, David!
    for l=0,n_max_b do begin
      if ((n+m+l) mod 2. eq 0.) then $
      c(n,m,l)=(2.*!pi)^.5*(-1.)^(1.5*n+.5*m+.5*l)*bthree(n,m,l)
    endfor
  endfor
endfor
; ...the following alternative, which gives exactly the same result, but takes slightly longer
;c_complex=complexarr(n_max_g+1,n_max_a+1,n_max_b+1)
;for n=0,n_max_g do begin
;  for m=0,n_max_a do begin
;    for l=0,n_max_b do begin
;      c_complex(n,m,l)=(2.*!dpi)^.5*(-1.)^n*complex(0,1)^(n+m+l)*bthree(n,m,l)
;    endfor
;  endfor
;endfor
;c=c_complex

; Construct 2-dimensional convolution matrix (PaperI eqn 52)
cc=fltarr(nn_g,nn_a,nn_b)
;cc=complexarr(nn_g,nn_a,nn_b)
for n=0,nn_g-1 do begin
  for m=0,nn_a-1 do begin
    for l=0,nn_b-1 do begin
      cc(n,m,l)=c(n1_g(n),n1_a(m),n1_b(l))*c(n2_g(n),n2_a(m),n2_b(l))
    endfor
  endfor
endfor

; Calculate the "PSF matrix" (PaperII eqn 12)
P=fltarr(nn_g,nn_a)
for n=0,nn_g-1 do begin
  for m=0,nn_a-1 do begin
    for l=0,nn_b-1 do begin
      P(n,m)=P(n,m)+cc(n,m,l)*g(l)
    endfor
  endfor
endfor

return,P

end
