pro shapelets_read_psf,decomp,fwhm,n_max=n_max,recomp=recomp,pixsize=pixsize,$
     snap=snap,hst=hst,diffsnap=diffsnap,silent=silent

;$Id: shapelets_read_psf.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Apr 03 - Variable pixel size from command line added by RM
;          This renders HDF,PC,ACS options obsolescent.
; Mar 03 - ACS option added by RM
; Nov 02 - Difference between SNAP and HST added by RM
;          (convolve HDF galaxies with this to get SNAP galaxies)
; Oct 02 - HST Planetary camera PSF added by RM
;          (same as HST WFPC, but with a different pixel scale)
; Jun 02 - SNAP PSF added by RM
; Apr 02 - HST WFPC (TinyTim) PSF incorporated by RM
; Apr 02 - Written by R.Massey
;
; PURPOSE:
; Creates a shapelet decomp structure representing a PSF.
;
; TO DO:
; Can I model the GAIA PSF?
;
; CALLING PROCEDURE:
; With no keywords, returns a unit-normalised Gaussian (00 state) with width
; "fwhm" pixels. eg "shapelets_read_psf,psf,4.0"
; With either /HST, /PC, /ACS, /SNAP or /GAIA, will read in a raytraced PSF 
;   from that telescope,decompose it into shapelets and then return that. 
;   NB: /HST is the WFPC L and /PC is the (higher resolution) Planetary Camera.
; A pixellised image of the PSF model is optionally returned in recomp.
;-

name=""                                         ; Default telescope name
if not keyword_set(pixsize) then pixsize=0.04   ; Default "/pixel

if keyword_set(hst) then begin
  
  ; Print intention to screen
  message,'Reading in TinyTim model of HST PSF',/info,noprint=silent
  
  ; Set pixel scale (for backwards compatability)
  if keyword_set(hdf) then pixsize=0.04  ; DITHERed HDF has 0.04"/pix
  if keyword_set(pc)  then pixsize=0.046 ; Planetary Camera (in centre) has 0.046"/pix 
  if keyword_set(acs) then pixsize=0.05  ; ACS wide field imager has 0.05"/pix 
  
  ; Read in HST TinyTim PSF image and decompose it into shapelets
  fits_read,shapelets_paths(4,silent=silent),psf; This has 0.01"/pix (10x oversampled WFPC)

  ; Set optimum parameters for decomposition
  x0=[150.50,150.48]
  if not keyword_set(n_max) then n_max=8
  beta=5. ; 3.89 minimises chisq but 5 also captures a bit of the 3rd diffraction wing
  shapelets_decomp,psf,beta,n_max,decomp1,recomp,x0=x0,/silent
  decomp1.beta=decomp1.beta/10.   ; Now has 0.1"/pix (regular WFPC)
  decomp1.n_pixels=[32,32]        ; Arbitrary image viewing parameters
  decomp1.x=[16,16]               ; 

  ; Add in effect of charge diffusion to neighbouring pixels
  shapelets_read_psf,ch_diff,.355,/silent
  decomp=shapelets_convolve(decomp1,ch_diff,gamma=0.6,n_max_g=n_max) 
  
  ; Set final pixel scale
  decomp.beta=decomp.beta*(0.1/pixsize)
  
  ; Renormalise to unity explicitly so that convolution conserves flux exactly
  shapelets_recomp,decomp,recomp
  weight=total(recomp) 
  decomp.coeffs(*)=decomp.coeffs(*)/weight
  recomp=recomp/weight
  decomp.name='HST TinyTim'

endif else if keyword_set(snap) then begin
  
  ; Print intention to screen
  message,'Reading in predicted model of SNAP PSF',/info,noprint=silent

  ; Read in Jason`s SNAP PSF and decompose it into shapelets
  fits_read,shapelets_paths(5,silent=silent),psf  ; This has 0.016 arcsec/pixel
  psf=psf[923:1122,923:1122]
  
  ; Set optimum parameters for decomposition
  x0=[99.95,101.5]
  if not keyword_set(n_max) then n_max=12
  w=5.3 ; 5.8 with nmax of 8, 7.5 with 12 to get 4th ring
  shapelets_decomp,psf,w,n_max,decomp,recomp,x0=x0,/silent
  decomp.beta=decomp.beta*(0.016/pixsize) ; }
  decomp.n_pixels=[30,30]                 ; } Image file was oversampled
  decomp.x=[15.5,15.5]                    ; }
 
  ; Renormalise so that convolution conserves flux
  shapelets_recomp,decomp,recomp
  weight=total(recomp)
  decomp.coeffs(*)=decomp.coeffs(*)/weight
  ;recomp=recomp/weight
  decomp.name='SNAP full'

endif else if keyword_set(diffsnap) then begin
  
  ; Print intention to screen
  message,'Calculating difference between HST and SNAP PSFs',/info,noprint=silent

  ; Read in Jason`s SNAP PSF model
  filename=shapelets_paths(5,silent=silent)
  fits_read,filename,psf  ; This has 0.016 arcsec/pixel
  psf=psf[923:1122,923:1122]
  
  ; Read in TinyTim model of HST PSF to deconvolve from the SNAP PSF
  shapelets_read_psf,hst,/hst,n_max=n_max,pixsize=0.016,/silent
 
  ; Set optimum parameters for decomposition
  x0=[99.95,101.5]
  if not keyword_set(n_max) then n_max=12
  w=4.8   ; This is still not really optimised!
  shapelets_decomp,psf,w,n_max,decomp,recomp,x0=x0,psf=hst,/silent
  decomp.beta=decomp.beta*(0.016/pixsize) ; }
  decomp.n_pixels=[30,30]                 ; } Image file was oversampled
  decomp.x=[15.5,15.5]                    ; }
  decomp.name='SNAP difference'
 
  ; Renormalise so that convolution conserves flux
  shapelets_recomp,decomp,recomp2
  weight=total(recomp2)
  decomp.coeffs(*)=decomp.coeffs(*)/weight
  recomp2=recomp2/weight

endif else begin
  
  ; Print intention to screen
  message,'Creating a Gaussian PSF',/info,noprint=silent

  ; Create a (unit normalised) Gaussian PSF - set FWHM << beta for a delta function
  n_max=0
  shapelets_make_nvec, n_max, n1, n2, n_coeffs
  coeffs=fltarr(n_coeffs) & coeffs[0]=.5/fwhm/sqrt(!pi) ; FWHM in pixels, so pixsize=1 by default
  n_pixels=max([fix(fwhm*5+.5),10])
  decomp = {name:"Gaussian_"+strtrim(string(fwhm),2), $
       	  type:"cartesian_decomp",      	         $
       	  history:strarr(1),	       	         $
       	  x:[n_pixels,n_pixels]/2., 	              $
       	  beta:fwhm, 			       	         $
       	  n_max:0,      		 	  	         $
       	  n_coeffs:n_coeffs,		  	         $
       	  coeffs:coeffs,		 	  	         $
       	  n_pixels:[n_pixels,n_pixels],             $
       	  moments:0B,  		 	  	         $
       	  sex:0B, 			 	  	         $
       	  n1:n1,  				  	         $ ; } Obsolescent
       	  n2:n2,  				  	         $ ; }
       	  over:1,			     	  	         $
       	  integrate:1,	          	  	         $
       	  error:fltarr(n_coeffs), 	  	         $
       	  chisq:[0.,0.], 		                   $
       	  skyfit:0}

  if keyword_set(recomp) then shapelets_recomp,decomp,recomp

endelse

end







