pro shapelets_sexcat2pstamp,image,sexcat,id,pstamp,$
    renorm=renorm, $                ; renormalise local noise level
    border=border, $                ; add an extra border to look pretty in plots
    silent=silent, plotit=plotit    ; verbose or not

;$Id: shapelets_image2pstamp.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_IMAGE2PSTAMP
;
; MODIFICATION HISTORY:
; Dec 02 - Border flag added by RM to make nice plots in papers. Not used elsewhere.
; Apr 02 - pstamp{}, image{} and sexcat{} structures formalism incorporated by RM.
; Dec 01 - Extract.pro written by R. Massey.
;
; PURPOSE: Extract a postage stamp of image data around an object in the
;          Hubble Deep Field and combine this with SExtractor data to
;          create an object structure. Uses the fact that the inverse variance
;          is zero on saturated stars and around the edge of the HDF.
;
; INPUT:  image: Image structure (eg. can be of whole HDF)
;        sexcat: SExtractor catalogue of image (read in with readsexcat.pro)
;            id: ID number of the object of interest in the SEx catalogue
;
; OUTPUT:   pstamp: Postage stamp structure
;
; TO DO: flagging is buggy - but anything above 4 should just be ignored anyway.
;
; flag:
; 0: OK
; 1: did not normalise noise locally (is object filling postage stamp?)
; 2: nearby object (beware overlapping isophotes!)
; 3: both of above
; 4: near edge or a saturated star
; 5: Zero FWHM: is this just some tatty noise?
; 6: HUGE object (SExtractor gaves whole image occasionally)
; NB can only be one of these. If duplicated, a higher number takes priority.
;-

on_error,2 & pstamp={flag:4} ; Default is to fail!
flag=0



; Guess at bounds of postage stamp using SExtractor information
;theta = (sexcat.theta(id))*!pi/180 ; a/c/w from x axis
;x=sexcat.x(id) & y=sexcat.y(id)
;a=3*sexcat.a(id) & b=3*sexcat.b(id)
;xmin   = round(x-abs(a*cos(theta))-abs(b*sin(theta))-3) > 0
;xmax   = round(x+abs(a*cos(theta))+abs(b*sin(theta))+2) < (image.npix(0)-1)
;ymin   = round(y-abs(a*sin(theta))-abs(b*cos(theta))-3) > 0
;ymax   = round(y+abs(a*sin(theta))+abs(b*cos(theta))+2) < (image.npix(1)-1)

; Actually want a square postage stamp, since the basis functions are circular:
; if they are big enough to encompass an elongated object, they must not fall
; off the edge in the direction of the minor axis.
; For low surface brightness galaxies, we need the whole FWHM, even though for
; bright objects we could make do with half of it.
nfwhm = 2.75 + keyword_set(border)
xmin  = round(sexcat.x(id,0)-nfwhm*sexcat.a(id)-5) > 0
xmax  = round(sexcat.x(id,0)+nfwhm*sexcat.a(id)+4) < (image.npix(0)-1)
ymin  = round(sexcat.x(id,1)-nfwhm*sexcat.a(id)-5) > 0
ymax  = round(sexcat.x(id,1)+nfwhm*sexcat.a(id)+4) < (image.npix(1)-1)


; Some objects in the HDF are just noise around the edges. SExtractor gives
; these zero FWHM
if sexcat.fwhm[id] eq 0. then begin
  flag=5
  message,'SExtractr thinks this object has zero FWHM! Noise?',/info,noprint=silent
endif

; Check noise (inverse variance) map for any troublesome objects nearby
bitn = image.noise[xmin:xmax,ymin:ymax]
if min(bitn) eq 0 then begin
  flag=4
  message,'Object near an edge or a saturated star!',/info,noprint=silent
  ;return
endif

; Extract region of interest
bit   = image.image[xmin:xmax,ymin:ymax]
bits  = image.seg[xmin:xmax,ymin:ymax]

; Check segmented pixel map for any regular objects nearby
othpix= where((bits ne 0) and (bits ne (id+1)),nothpix)
obpix = where((bits eq (id+1)),nobpix) ; This had better be >0 !
if nothpix then begin
   flag=flag+2
;  message,'Masking out a nearby object',/info,noprint=silent
;  bitn[othpix]=0.   ; Set weight of those pixels to zero
endif

; Normalise noise map locally
bgpix = where(bits eq 0,nbgpix)
if keyword_set(renorm) and nbgpix gt 10 then begin
  nmnorm=(stddev(bit[bgpix]))^(-2) / mean(bitn[bgpix]) 
  ;nmnorm=(stddev(bit[bgpix])*1.9)^(-2) / mean(bitn[bgpix]) ; 1.9 from Williams et al 1996
  ;if (strmid(sexcat.name,0,3) eq "HDF") then nmnorm=nmnorm/0.28 ; from Williams et al 1996
  if (strmid(sexcat.name,0,3) eq "HDF") then nmnorm=nmnorm/0.33 ; from Williams et al 1996
  message,'Local normalisation of noise map: '+$
    strtrim(nmnorm,1),/info,noprint=silent
  bitn=bitn*nmnorm
endif else flag=flag+1

; Convert centroid guess into postage-stamp coordinates
x0=[sexcat.x(id,0),sexcat.x(id,1)]-[xmin,ymin]


; Store in an "object" structure
pstamp={name:image.name+"_#"+strtrim(id,2),$
        type:"pstamp",           $
        flag:flag,               $      ; 0=fail, 1=OK, 2=Nearby obj, 4=Noise normalised
        image:bit,               $      ; Image data,
        noise:bitn,              $      ; Pixel weight map
        seg:bits,                $      ; Image segmentation map
        im_ran:[xmin,xmax,ymin,ymax],$  ; Coordinates of BL and TR of postage stamp
        xo:x0(0),                $      ; Centroid guess in postage stamp coordinate
        yo:x0(1),                $      ;
        x:sexcat.x(id,0),        $      ; SEx centroid guess in image coordinates
        y:sexcat.x(id,1),        $      ;
        xpeak:sexcat.xpeak(id,0),$      ;
        ypeak:sexcat.xpeak(id,1),$      ;
        xmin:sexcat.xmin(id,0),  $      ;
        ymin:sexcat.xmin(id,1),  $      ;
        xmax:sexcat.xmax(id,0),  $      ;
        ymax:sexcat.xmax(id,1),  $      ; Other SExtractor
        area:sexcat.area(id),    $      ;  parameters, all in
        a:sexcat.a(id),          $      ;  image coordinates
        b:sexcat.b(id),          $      ;
        theta:sexcat.theta(id),  $      ;
        class:sexcat.class(id),  $      ;
        nu:sexcat.flux(id),      $      ;
        fwhm:sexcat.fwhm(id),    $      ;
        mag:sexcat.mag(id)}             ;




;
;
; Plot our postage stamp and the SExtractor parameters
;
;
if keyword_set(plotit) then begin
  window,1,xsize=500,ysize=555,xpos=640,ypos=350
  ; Image
  ;bit[othpix]=0.
  ;bit=bit*sqrt(bitn)
  plt_image,bit,/frame,/col,title='!6Postage stamp #'+strtrim(id,1)
  ; Crosshairs
  oplot,[x0[0],x0[0]],[0,300],linestyle=2
  oplot,[0,300],[x0[1],x0[1]],linestyle=2
  ; SExtractor ellipse
  plt_ellipse,x0(0),x0(1),2.*sexcat.a(id),2.*sexcat.b(id),(sexcat.theta(id)+90)*!pi/180,linestyle=0
  plt_ellipse,x0(0),x0(1),sexcat.fwhm(id),sexcat.fwhm(id),0,linestyle=1
  ; SExtractor min/max pixels
  plt_rectangle,[sexcat.xmin(id,0)-xmin,sexcat.xmin(id,1)-ymin], $
            [sexcat.xmax(id,0)-xmin,sexcat.xmax(id,1)-ymin],linestyle=2
endif



end
